function hex_dock(savename,a,lat,d)

w = 18;     % Hexamer diameter measured from the hinge (nm)
w1 = 21;    % Hexamer diameter measured from Fabs (nm)
h = 12;     % Height of F above the membrane (nm)
a0 = 5;     % Fab binding radius (nm)
delta = 10; % Thickness of the region where Fcs are randomly placed (nm)
L = 50;     % Length of each side of the concentric patch over which hexamer is randomly placed
show = 0;   % Set equal to 1 to display accepted IgG hexamer configurations

fc = [0 w/2 6 ;
      w/2*sin(pi/3) w/2*cos(pi/3) 6 ;
      w/2*sin(2*pi/3) w/2*cos(2*pi/3) 6 ;
      w/2*sin(pi) w/2*cos(pi) 6 ;
      w/2*sin(4*pi/3) w/2*cos(4*pi/3) 6 ;
      w/2*sin(5*pi/3) w/2*cos(5*pi/3) 6];
fab = [0 w1/2 0 ;
       w1/2*sin(pi/3) w1/2*cos(pi/3) 0 ;
       w1/2*sin(2*pi/3) w1/2*cos(2*pi/3) 0 ;
       w1/2*sin(pi) w1/2*cos(pi) 0 ;
       w1/2*sin(4*pi/3) w1/2*cos(4*pi/3) 0 ;
       w1/2*sin(5*pi/3) w1/2*cos(5*pi/3) 0];

% % Cylindrical curvature, upright square lattice:
if lat == 1
  m1 = [-5/2 -3/2 -1/2 1/2 3/2 5/2]; m2 = [-5/2 -3/2 -1/2 1/2 3/2 5/2];
  [xx,yy] = meshgrid(m1,m2);
  Fx = (a+h)*sin(xx*d/a);     Fy = yy*d;      Fz = (a+h)*cos(xx*d/a);
  F0 = [Fx(:) Fy(:) Fz(:)];
end

% % Cylindrical curvature, diagonal square lattice:
if lat == 2;
  m1 = [-5/2 -3/2 -1/2 1/2 3/2 5/2]*sqrt(2);
  m2 = [-5/2 -3/2 -1/2 1/2 3/2 5/2]*sqrt(2);
  [xx,yy] = meshgrid(m1,m2);
  Fx = (a+h)*sin(xx*d/a);     Fy = yy*d;      Fz = (a+h)*cos(xx*d/a);
  F0 = [Fx(:) Fy(:) Fz(:)];
  m1 = [-2 -1 0 1 2]*sqrt(2);
  m2 = [-2 -1 0 1 2]*sqrt(2);
  [xx,yy] = meshgrid(m1,m2);
  Fx = (a+h)*sin(xx*d/a);     Fy = yy*d;      Fz = (a+h)*cos(xx*d/a);
  F1 = [Fx(:) Fy(:) Fz(:)]; F0 = [F0 ; F1];
end

% Cylindrical curvature, hexagonal lattice:
if lat == 3;
  m1 = [-5/2 -3/2 -1/2 1/2 3/2 5/2];
  m2 = [-5/2 -3/2 -1/2 1/2 3/2 5/2]*sqrt(3);
  [xx,yy] = meshgrid(m1,m2);
  Fx = (a+h)*sin(xx*d/a);     Fy = yy*d;      Fz = (a+h)*cos(xx*d/a);
  F0 = [Fx(:) Fy(:) Fz(:)];
  m1 = [-2 -1 0 1 2];
  m2 = [-2 -1 0 1 2]*sqrt(3);
  [xx,yy] = meshgrid(m1,m2);
  Fx = (a+h)*sin(xx*d/a);     Fy = yy*d;      Fz = (a+h)*cos(xx*d/a);
  F1 = [Fx(:) Fy(:) Fz(:)]; F0 = [F0 ; F1];
end

% Plot the membrane:
m1 = [-3:.1:3]; m2 = [-3:.1:3]*sqrt(3);
[xx,yy] = meshgrid(m1,m2);
mx = a*sin(xx*d/a);     my = yy*d;      mz = a*cos(xx*d/a);
m0 = [mx(:) my(:) mz(:)]; 
m1 = [-3.05:.1:3]; m2 = [-3.05:.1:3]*sqrt(3);
[xx,yy] = meshgrid(m1,m2);
mx = a*sin(xx*d/a);     my = yy*d;      mz = a*cos(xx*d/a);
m01 = [mx(:) my(:) mz(:)]; m0 = [m0 ; m01]; 
m0(:,3) = m0(:,3) - max(F0(:,3));

% Initialize the hexamer configurations:
K = 2000000;
XYZ(:,1) = 2*L*(rand(50*K,1)-0.5);
XYZ(:,2) = 2*L*(rand(50*K,1)-0.5);
XYZ(:,3) = 2*L*(rand(50*K,1)-0.5) + (a+h);

% Remove points that are outside the concentric patch we are testing:
i = find((XYZ(:,2)<L/2).*(XYZ(:,2)>-L/2)); XYZ = XYZ(i,:);
i = find(XYZ(:,3)>0); XYZ = XYZ(i,:);
R = sqrt(XYZ(:,1).^2 + XYZ(:,3).^2);
i = find((R>(a+h)).*(R<(a+h+delta)));
XYZ = XYZ(i,:); R = R(i);
H0 = L/(a+h);
hh = asin(XYZ(:,1)./R);
i = find((hh<H0/2).*(hh>-H0/2));
XYZi = XYZ(i(1:K),:);

% Adjust z-coordinate so the the position of the highest epitope = 0.
XYZi(:,3) = XYZi(:,3) - max(F0(:,3));
F0(:,3) = F0(:,3) - max(F0(:,3));

% % Use the lines below to plot the sampled IgG center positions relative to the epitopes:
% plot3(F0(:,1),F0(:,2),F0(:,3),'ob','MarkerSize',24,'MarkerFaceColor',[0.3 0.5 1]); hold on; axis equal
% plot3(m0(:,1),m0(:,2),m0(:,3),'o','MarkerSize',6,'MarkerFaceColor',[0.6 0.6 0.6],'MarkerEdgeColor',[0.6 0.6 0.6]);
% plot3(XYZi(:,1),XYZi(:,2),XYZi(:,3),'k.'); drawnow; hold off;
% return

X = []; T = [];
for k = 1:K;
  t1 = pi/2*(rand(1)-.5);
  t2 = pi/2*(rand(1)-.5);
  t3 = pi/3*rand(1);
  XYZ = XYZi(k,:);

R1 = [1 0 0;
      0 cos(t1) -sin(t1);
      0 sin(t1) cos(t1)];
R2 = [cos(t2) 0 sin(t2) ;
      0 1 0 ;
      -sin(t2) 0 cos(t2)];
R3 = [cos(t3) -sin(t3) 0;
      sin(t3) cos(t3) 0;
      0 0 1];
R = R1*R2*R3;
fc1 = (R*fc')' + ones(size(fc,1),1)*XYZ;
fab1 = (R*fab')' + ones(size(fc,1),1)*XYZ;

% Check for overlap between Fabs and antigen positions:
dx = fab1(:,1)*ones(size(F0(:,1)))' - ones(size(fab1(:,1)))*F0(:,1)';
dy = fab1(:,2)*ones(size(F0(:,2)))' - ones(size(fab1(:,1)))*F0(:,2)';
dz = fab1(:,3)*ones(size(F0(:,3)))' - ones(size(fab1(:,1)))*F0(:,3)';
dr = sqrt(dx.^2 + dy.^2 + dz.^2);
dr_min = min(dr,[],2);              
S(k) = sum(dr_min<a0);

% If the shortest distance between each Fab and any epitope is less than
% a0, store the position.  If 'show' set to 1, display the configuration.
if(sum(dr_min<a0)==6)
    X = [X ; XYZ];
    T = [T ; t1 t2 t3];
    if(show==1)
      for q = 1:6
        [val,i] = min(dr(q,:));
        plot3([fab1(q,1) F0(i,1)],[fab1(q,2) F0(i,2)],[fab1(q,3) F0(i,3)],'k:','LineWidth',2); hold on;
      end

      plot3(F0(:,1),F0(:,2),F0(:,3),'ob','MarkerSize',24,'MarkerFaceColor',[0.3 0.5 1]); hold on; axis equal
      plot3(m0(:,1),m0(:,2),m0(:,3),'o','MarkerSize',6,'MarkerFaceColor',[0.6 0.6 0.6],'MarkerEdgeColor',[0.6 0.6 0.6]);
      xlim([-30 30]); ylim([-30 30]); zlim([-30 30]);
      fill3(fc1(:,1),fc1(:,2),fc1(:,3),[1 0.8 0.8]);
      plot3(fab1(:,1),fab1(:,2),fab1(:,3),'o','MarkerSize',36,'MarkerFaceColor',[1 0.5 .5],'MarkerEdgeColor',[1 0.5 .5]/2);
      drawnow; hold off;
    end
  end
end

disp(['Accepted ',num2str(size(X,1)),' out of ',num2str(length(S)),' configurations.'])
save(savename,'X','T','S','fc','fab'); 